@@ -1,13 +1,12 @@
 """Classes for producing HTML reports about imprecision."""
 
 from abc import ABCMeta, abstractmethod
-import cgi
 import json
 import os
 import shutil
 import tokenize
 from operator import attrgetter
-
+from urllib.request import pathname2url
 from typing import Any, Callable, Dict, List, Optional, Tuple, cast
 
 import time
@@ -144,6 +143,74 @@ class LineCountReporter(AbstractReporter):
 register_reporter('linecount', LineCountReporter)
 
 
+class AnyExpressionsReporter(AbstractReporter):
+    def __init__(self, reports: Reports, output_dir: str) -> None:
+        super().__init__(reports, output_dir)
+        self.counts = {}  # type: Dict[str, Tuple[int, int]]
+        stats.ensure_dir_exists(output_dir)
+
+    def on_file(self,
+                tree: MypyFile,
+                type_map: Dict[Expression, Type],
+                options: Options) -> None:
+        visitor = stats.StatisticsVisitor(inferred=True, filename=tree.fullname(),
+                                          typemap=type_map, all_nodes=True)
+        tree.accept(visitor)
+        num_unanalyzed_lines = list(visitor.line_map.values()).count(stats.TYPE_UNANALYZED)
+        # count each line of dead code as one expression of type "Any"
+        num_any = visitor.num_any + num_unanalyzed_lines
+        num_total = visitor.num_imprecise + visitor.num_precise + num_any
+        if num_total > 0:
+            self.counts[tree.fullname()] = (num_any, num_total)
+
+    def on_finish(self) -> None:
+        total_any = sum(num_any for num_any, _ in self.counts.values())
+        total_expr = sum(total for _, total in self.counts.values())
+        total_coverage = 100.0
+        if total_expr > 0:
+            total_coverage = (float(total_expr - total_any) / float(total_expr)) * 100
+
+        any_column_name = "Anys"
+        total_column_name = "Exprs"
+        file_column_name = "Name"
+        total_row_name = "Total"
+        coverage_column_name = "Coverage"
+        # find the longest filename all files
+        name_width = max([len(file) for file in self.counts] +
+                         [len(file_column_name), len(total_row_name)])
+        # totals are the largest numbers in their column - no need to look at others
+        min_column_distance = 3  # minimum distance between numbers in two columns
+        any_width = max(len(str(total_any)) + min_column_distance, len(any_column_name))
+        exprs_width = max(len(str(total_expr)) + min_column_distance, len(total_column_name))
+        coverage_width = len(coverage_column_name) + min_column_distance
+        header = '{:{name_width}} {:>{any_width}} {:>{total_width}} {:>{coverage_width}}'.format(
+            file_column_name, any_column_name, total_column_name, coverage_column_name,
+            name_width=name_width, any_width=any_width, total_width=exprs_width,
+            coverage_width=coverage_width)
+
+        with open(os.path.join(self.output_dir, 'any-exprs.txt'), 'w') as f:
+            f.write(header + '\n')
+            separator = '-' * len(header) + '\n'
+            f.write(separator)
+            coverage_width -= 1  # subtract one for '%'
+            for file in sorted(self.counts):
+                (num_any, num_total) = self.counts[file]
+                coverage = (float(num_total - num_any) / float(num_total)) * 100
+                f.write('{:{name_width}} {:{any_width}} {:{total_width}} '
+                        '{:>{coverage_width}.2f}%\n'.
+                        format(file, num_any, num_total, coverage, name_width=name_width,
+                               any_width=any_width, total_width=exprs_width,
+                               coverage_width=coverage_width))
+            f.write(separator)
+            f.write('{:{name_width}} {:{any_width}} {:{total_width}} {:>{coverage_width}.2f}%\n'
+                    .format(total_row_name, total_any, total_expr, total_coverage,
+                            name_width=name_width, any_width=any_width, total_width=exprs_width,
+                            coverage_width=coverage_width))
+
+
+register_reporter('any-exprs', AnyExpressionsReporter)
+
+
 class LineCoverageVisitor(TraverserVisitor):
     def __init__(self, source: List[str]) -> None:
         self.source = source
@@ -320,7 +387,8 @@ class MemoryXmlReporter(AbstractReporter):
         if 'stubs' in path.split('/'):
             return
 
-        visitor = stats.StatisticsVisitor(inferred=True, typemap=type_map, all_nodes=True)
+        visitor = stats.StatisticsVisitor(inferred=True, filename=tree.fullname(),
+                                          typemap=type_map, all_nodes=True)
         tree.accept(visitor)
 
         root = etree.Element('mypy-report-file', name=path, module=tree._fullname)
@@ -334,11 +402,11 @@ class MemoryXmlReporter(AbstractReporter):
                 etree.SubElement(root, 'line',
                                  number=str(lineno),
                                  precision=stats.precision_names[status],
-                                 content=line_text[:-1])
+                                 content=line_text.rstrip('\n'))
         # Assumes a layout similar to what XmlReporter uses.
         xslt_path = os.path.relpath('mypy-html.xslt', path)
         transform_pi = etree.ProcessingInstruction('xml-stylesheet',
-                'type="text/xsl" href="%s"' % cgi.escape(xslt_path, True))
+                'type="text/xsl" href="%s"' % pathname2url(xslt_path))
         root.addprevious(transform_pi)
         self.schema.assertValid(doc)
 
@@ -361,7 +429,7 @@ class MemoryXmlReporter(AbstractReporter):
                              module=file_info.module)
         xslt_path = os.path.relpath('mypy-html.xslt', '.')
         transform_pi = etree.ProcessingInstruction('xml-stylesheet',
-                'type="text/xsl" href="%s"' % cgi.escape(xslt_path, True))
+                'type="text/xsl" href="%s"' % pathname2url(xslt_path))
         root.addprevious(transform_pi)
         self.schema.assertValid(doc)
 
@@ -425,7 +493,8 @@ class CoberturaXmlReporter(AbstractReporter):
                 type_map: Dict[Expression, Type],
                 options: Options) -> None:
         path = os.path.relpath(tree.path)
-        visitor = stats.StatisticsVisitor(inferred=True, typemap=type_map, all_nodes=True)
+        visitor = stats.StatisticsVisitor(inferred=True, filename=tree.fullname(),
+                                          typemap=type_map, all_nodes=True)
         tree.accept(visitor)
 
         class_name = os.path.basename(path)
