@@ -32,8 +32,7 @@ from mypy.util import try_find_python2_interpreter
 python_eval_files = ['pythoneval.test',
                      'python2eval.test']
 
-python_34_eval_files = ['pythoneval-asyncio.test',
-                        'pythoneval-enum.test']
+python_34_eval_files = ['pythoneval-asyncio.test']
 
 # Path to Python 3 interpreter
 python3_path = sys.executable
@@ -85,14 +84,8 @@ def test_python_evaluation(testcase: DataDrivenTestCase) -> None:
     # This uses the same PYTHONPATH as the current process.
     returncode, out = run(mypy_cmdline)
     if returncode == 0:
-        # Set up module path for the execution.
-        # This needs the typing module but *not* the mypy module.
-        vers_dir = '2.7' if py2 else '3.2'
-        typing_path = os.path.join(testcase.old_cwd, 'lib-typing', vers_dir)
-        assert os.path.isdir(typing_path)
-        env = os.environ.copy()
-        env['PYTHONPATH'] = typing_path
-        returncode, interp_out = run([interpreter, program], env=env)
+        # Execute the program.
+        returncode, interp_out = run([interpreter, program])
         out += interp_out
     # Remove temp file.
     os.remove(program_path)
